
# Forecasts_quarterly.m #


# Loading data and defining variables, 1926:4-2010:4#

library(readxl)
library(openxlsx)
#library(xlsx)
source('nchoosek.r')
source('zscore.r')
source('perform_asset_allocation.r')
# Equity premium 

market_return = read_xlsx('dataset.xlsx', sheet = 'Feuil1', range = 'p3:p149', col_names = FALSE)
#write.table(market_return, file = 'market_return.csv', col.names = FALSE, row.names = FALSE)
r_f_lag = read_xlsx('dataset.xlsx', sheet = 'Feuil1', range = 'k2:k148', col_names = FALSE) #risk-free rate, lagged (1986:1-2022:4)
#write.table(r_f_lag, file = 'r_f_lag.csv', col.names = FALSE, row.names = FALSE)
equity_premium = log(1 + market_return) - log(1 + r_f_lag) #log excess return

# Predictors

D4 = read_xlsx('dataset.xlsx',sheet = 'Feuil1', range = 'c3:c149', col_names = FALSE) #dividends
#write.table(D4, file = 'D4.csv', col.names = FALSE, row.names = FALSE)
SP500 = read_xlsx('dataset.xlsx', sheet = 'Feuil1', range = 'b3:b149', col_names = FALSE) #Nasdaq 100 index
#write.table(SP500, file = 'SP500.csv', col.names = FALSE, row.names = FALSE)
DP = log(D4) - log(SP500) #log dividend-price ratio
SP500_lag = read_xlsx('dataset.xlsx', sheet = 'Feuil1', range = 'b2:b148', col_names = FALSE) #Nasdaq 100 index, lagged (1986:1-2022:4)
#write.table(SP500_lag, file = 'SP500_lag.csv', col.names = FALSE, row.names = FALSE)
DY = log(D4) - log(SP500_lag) #log dividend yield
E4 = read_xlsx('dataset.xlsx', sheet = 'Feuil1',range = 'd3:d149', col_names = FALSE) #earnings
#write.table(E4, file = 'E4.csv', col.names = FALSE, row.names = FALSE)
EP = log(E4) - log(SP500) #log earnings-price ratio
DE = log(D4) - log(E4) #log dividend-payout ratio
SVAR = read_xlsx('dataset.xlsx',sheet = 'Feuil1', range = 'o3:o149', col_names = FALSE) #volatility
#write.table(SVAR, file = 'SVAR.csv', col.names = FALSE, row.names = FALSE)
BM = read_xlsx('dataset.xlsx',sheet = 'Feuil1', range = 'e3:e149', col_names = FALSE) #book-to-market ratio
#write.table(BM, file = 'BM.csv', col.names = FALSE, row.names = FALSE)
NTIS = read_xlsx('dataset.xlsx',sheet = 'Feuil1', range = 'j3:j149', col_names = FALSE) #net equity issuing activity
#write.table(NTIS, file = 'NTIS.csv', col.names = FALSE, row.names = FALSE)
TBL = read_xlsx('dataset.xlsx', sheet = 'Feuil1', range = 'f3:f149', col_names = FALSE) #T-bill rate
#write.table(TBL, file = 'TBL.csv', col.names = FALSE, row.names = FALSE)
LTY = read_xlsx('dataset.xlsx', sheet = 'Feuil1',  range = 'i3:i149', col_names = FALSE) #long-term government bond yield
#write.table(LTY, file = 'LTY.csv', col.names = FALSE, row.names = FALSE)
LTR = read_xlsx('dataset.xlsx', sheet = 'Feuil1', range = 'm3:m149', col_names = FALSE) #long-term government bond return
#write.table(LTR, file = 'LTR.csv', col.names = FALSE, row.names = FALSE)
TMS = LTY - TBL #term spread
AAA = read_xlsx('dataset.xlsx',sheet = 'Feuil1', range = 'g3:g149', col_names = FALSE) #AAA-rated corporate bond yield
#write.table(AAA, file = 'AAA.csv', col.names = FALSE, row.names = FALSE)
BAA = read_xlsx('dataset.xlsx',sheet = 'Feuil1', range = 'h3:h149', col_names = FALSE) #BAA-rated corporate bond yield
#write.table(BAA, file = 'BAA.csv', col.names = FALSE, row.names = FALSE)
DFY = BAA - AAA #default yield spread
CORPR = read_xlsx('dataset.xlsx', sheet = 'Feuil1', range = 'n3:n149', col_names = FALSE) #long-term corporate bond return
#write.table(CORPR, file = 'CORPR.csv', col.names = FALSE, row.names = FALSE)
DFR = CORPR - LTR #default return spread
INFL_lag = read_xlsx('dataset.xlsx', sheet = 'Feuil1', range = 'l2:l148', col_names = FALSE) #inflation, lagged (1986:1-2022:4)
#write.table(INFL_lag, file = 'INFL_lag.csv', col.names = FALSE, row.names = FALSE)
ECON = cbind(DP, DY, EP, DE, SVAR, BM, NTIS, TBL, LTY, LTR, TMS, DFY, DFR, INFL_lag)
ECON_sink = cbind(DP, DY, EP, SVAR, BM, NTIS, TBL, LTY, LTR, DFY, DFR, INFL_lag)

#Sum-of-the-parts variables

E4_lag = read_xlsx('dataset.xlsx',sheet = 'Feuil1', range = 'd2:d148', col_names = FALSE) #earnings, lagged (1986:1-2022:4)
#write.table(E4_lag, 'E4_lag.csv', col.names = FALSE, row.names = FALSE)
E_growth=(E4-E4_lag)/E4_lag #earnings growth
DP_SOP=(1/4)*D4/SP500 # D/P
r_f = read_xlsx('dataset.xlsx', sheet = 'Feuil1', range= 'k3:k149', col_names = FALSE) #risk-free rate
#write.table(r_f, 'r_f.csv', col.names = FALSE, row.names = FALSE)


# Estimating full-sample parameters for Campbell-Thompson restrictions #


beta_full = matrix(0, ncol(ECON),1)#creating a matrix to store the regression coeff
T = nrow(equity_premium)
LHS = equity_premium[2:T, 1]#Left Hand Side, represents the depend var in this regression model, ie the equity premium
for (i in 1:ncol(ECON)){

    RHS_i = ECON[1:T-1,i]
    results_i = lm( LHS ~ RHS_i);
    beta_full[i] = results_i$coef[2];
   
}
beta_full[5] = 1 # restricting SVAR slope coefficient to be positive

# NB: following Campbell and Thompson (2008) by using full-sample
# estimates as theoretical 'priors' (although we make sure that
# SVAR slope coefficient is positive)


# Preliminaries #


Y = equity_premium
T = nrow(Y)
N = ncol(ECON)
R = (2001-1986) * 4 - 1 #in-sample period, 1986:4-2001:4
P_0 = (2022 - 2002) * 4 - 4# holdout out-of-sample period, 2002:1-2022:4
P = T - (R + P_0) #forecast evaluation period
theta = 0.9 # discount factor for DMSPE pooled forecast
r = 1 #number of principal components
MA_SOP = 20 * 4 #window size for SOP forecast


# Computing forecasts, 1986:1-2022:4 (includes holdout OOS period) #



FC_HA = matrix(0, P_0 + P, 1) #matrix to store historical average forecasts
FC_ECON = matrix(0, P_0 + P, N)#stores forecast from individual predictive regressions for each predictor
beta_ECON = array(0,  dim = c(P_0 + P , N , 2)) #3 dimension array to store coefficients and standard errors from the regressions
#N corresponds to the number of predictors in the ECON dataset
#2 is to store both the coeff and its standard error for each predictor and time period
FC_ECON_CT = matrix(0, P_0 + P ,N) #FC_ECON under Campbell-Thompson restrictions
omega_DMSPE = matrix(0, P_0 + P, N)#store weights for DMSPE pool forecasts
FC_OTHER = matrix(0, P_0 + P, 5 + length(theta))#store from other models
FC_OTHER_CT = matrix(0, P_0 + P, 5 + length(theta))#FC_OTHER under Campbell-Thompson restrictions
for (t in 1:(P_0 + P)){
  
  
  FC_HA[t]=mean(Y[1:(R+(t-1)), 1])
  #historical average serves as a simple, non-parametric benchmark forecast
  #the loop dynamically updates to inclure more data as t increase
  #the later periods use more historical information

  # Individual predictive regression forecasts
  
  X_t=ECON[1:(R+(t-1)-1),]#dynamically updates as t increments
  Y_t=Y[2:(R+(t-1)), 1];#shifted by one period to align with the lagged predictors
  for(i in 1:N){
    results_t_i = lm (Y_t ~ X_t[,i]);#linear model is trained to fit the historical data
    FC_ECON[t,i] = c(1, ECON[(R+(t-1)),i]) %*% results_t_i$coefficients#generating forecasts using the estimated coefficients and the current values of the predictors
    beta_ECON[t,i,1] = summary(results_t_i)$coefficients[2, 1]
    beta_ECON[t,i,2] = summary(results_t_i)$coefficients[2, 2]
    #beta coeff and their standard errors are stored 
    
    #Campbell Thompson restrictions
    if (beta_full[i] > 0){
      if (summary(results_t_i)$coefficients[2, 1] > 0){
        FC_ECON_CT[t, i] = FC_ECON[t, i]
      }else if (summary(results_t_i)$coefficients[2, 1] < 0){
        FC_ECON_CT[t, i] = FC_HA[t]
      }
    }else if (beta_full[i] < 0){
      if (summary(results_t_i)$coefficients[2, 1] < 0){
        FC_ECON_CT[t, i] = FC_ECON[t, i]
      }else if (summary(results_t_i)$coefficients[2, 1] > 0){
        
        
        FC_ECON_CT[t, i]=FC_HA[t]
      }
    }
    if (FC_ECON_CT[t, i]< 0){
      FC_ECON_CT[t, i] = 0;
    }
    #If a beta is expected to be positive, but the estimate is negative, the forecast from the historical is used instead
    #Negative forecasts are set to zero
  }
  #Continuation condition, to start a new calculation only after all out-of-sample period are processed
  if (t > P_0){
    
    #Kitchen sink forecast
    
    X_t_sink = ECON_sink[1:(R+(t-1)-1),]#extract predictors from ECON_sink
    temp_var_for_lm = cbind(Y_t, X_t_sink)#combining the equity premium and the predictors into a single data frame to fit a linear model
    colnames(temp_var_for_lm) <- c('y', paste0('x', seq(ncol(X_t_sink))))
    results_t_sink=lm(as.formula(sprintf("y ~ %s", paste0(names(temp_var_for_lm)[2:ncol(temp_var_for_lm)], collapse = " + "))), data = temp_var_for_lm)
    # linear model is used through the lm function
    FC_OTHER[t, 1] = t(c(1, unlist(ECON_sink[(R+(t-1)),])) ) %*% results_t_sink$coefficients;
    #calculating the forecasts for the current period t by multiplying the transpose of the vector containing a constant term, for intercept
    # and current values of predictors by the coefficients obtained from the regression
    if (FC_OTHER[t, 1] < 0){
      FC_OTHER_CT[t, 1] = 0
    }else{
      FC_OTHER_CT[t, 1] = FC_OTHER[t, 1];
    }
    #If FC_other[t,1] is negative, FC_OTHER_CT is set to zero. If forecast not negative, we use FC_OTHER_CT
    
    #Forecasting is comprehensive and utilizes a large set of predictors, which can be beneficial
    #in capturing complex relationships but also risky in terms of overfitting and model complexity
    
    
    #SIC forecast (Schwarz Information Criterion)
    
    j_max=3 #consider models with up to 3 predictors to save time and complexity
    SIC_t=data.frame(matrix(vector(), 0, 2));
    ind = 1
    for (j in 1:j_max){
      select_j=as.matrix(nchoosek(1:1:ncol(X_t_sink),j), ncol = 1)
      for (k in 1:nrow(select_j)){
        X_t_j_k=data.frame(X_t_sink[1:(R+(t-1)-1), select_j[k,]])
        #results_t_j_k=lm(Y_t ~ X_t_j_k);
        
        temp_var_for_lm2= data.frame(cbind(Y_t, X_t_j_k))
        colnames(temp_var_for_lm2) <- c('y', paste0('x', seq(ncol(X_t_j_k))))
        results_t_j_k=lm(as.formula(sprintf("y ~ %s", paste0(names(temp_var_for_lm2)[2:ncol(temp_var_for_lm2)], collapse = " + "))), data = temp_var_for_lm2)
        SIC_t_j_k=log(t(results_t_j_k$resid) %*% results_t_j_k$resid / length(Y_t))+log(length(Y_t)) * ncol(X_t_j_k)/length(Y_t);
        FC_t_j_k=t(c(1, unlist(ECON_sink[(R+(t-1)), select_j[k,]]))) %*% results_t_j_k$coefficients;
        SIC_t[ind, 1] = SIC_t_j_k 
        SIC_t[ind, 2] =  FC_t_j_k
        ind = ind + 1
      }
      #for each combination, a linear model is fitted using the selected predictors
      #The SIC balances model fit and complexity by penalizing models with more parameters
    }
    SIC_t_min = min(SIC_t[,1])
    SIC_t_min_index = which.min(SIC_t[,1]) #the model with the lowest SIC value is selected as the best
    FC_OTHER[t, 2]=SIC_t[SIC_t_min_index, 2]
    if (FC_OTHER[t, 2] < 0){
      FC_OTHER_CT[t, 2] = 0
    }else{
      FC_OTHER_CT[t, 2] = FC_OTHER[t, 2]
    }
    #negative forecasts are set to zero in FC_OTHER_CT
    
    # This approach aims to find the best balance between fit and simplicity, without risk of overfitting
    
    
    #Pooled forecast: simple average
    
    FC_OTHER[t, 3] = mean(FC_ECON[ t,])
    if (FC_OTHER[t, 3] < 0){ #ensure that forecasts are non-negative
      FC_OTHER_CT[t,3]=0;
    }else{
      FC_OTHER_CT[t,3]=FC_OTHER[t,3];
    }
    # leveraging the strengths of multiple models by averaging their forecasts
    # Assume equal reliability across models
    #Pooled forecast: DMSPE
    
    powers_t=matrix(seq(t-2,0), nrow = 1)#apply the discount factor to more recent errors more heavily
    m=matrix(colSums((
      
      kronecker(matrix(rep(1,N), nrow = 1), #use to expand the weighting scheme across all forecasts and all periods up to t-1
                (theta*matrix(rep(1, t-1), ncol = 1))^t(powers_t))
    )*(
      (
        kronecker(
          matrix(rep(1,N), nrow = 1),Y[(R+1):(R+(t-1)), 1])
        -FC_ECON[1:(t-1),]
      )^2
    )
    ), ncol = 1)
    omega=(m^(-1))/(sum(m^(-1)))
    # omega normalized weights to ensure they sum to 1
    # m calculates the sum of weighted squaared forecasts errors for each model
    omega_DMSPE[t,] = t(omega)
    
    FC_OTHER[t, 4] = matrix(FC_ECON[ t,], nrow = 1) %*% matrix(omega, ncol =1)#focusing more on models that have performed better recently
    if (FC_OTHER[t, 4] < 0){
      FC_OTHER_CT[t, 4] = 0
    }else{
      FC_OTHER_CT[t, 4] = FC_OTHER[t, 4]
    }
    #Non negative
    
    #leveraging the strengths of multiple models by averaging their forecasts
    #DMSPE adapts weights based on recent performance, potentially providing a more accurate method in changing economic conditions
    
    #Diffusion index forecast (PCA- principal component analysis)
    
    X_t_DI_standardize = apply(ECON[1:(R+(t-1)),], 2, zscore);#standardization through z-score
    my_var_for_pca = princomp(X_t_DI_standardize);#reduce dimensionality of the data while attempting to preserve as much variation as possible
    Lambda_t = my_var_for_pca$loadings#store the coefficients of the principal components
    F_t = my_var_for_pca$scores
    results_t_DI = lm(Y_t ~ F_t[1:(nrow(F_t)-1),1:r] );#r is predefined, limiting the model to the component that captures the most variance
    FC_OTHER[t, 5] = c(1, F_t[nrow(F_t), 1:r]) %*% results_t_DI$coefficients;#applying regression coefficients to the principal components for the current period
    if (FC_OTHER[t, 5] < 0){
      FC_OTHER_CT[t, 5] = 0;
    }else{
      FC_OTHER_CT[t,5]=FC_OTHER[t,5];
    };
    #non-negativity
    
    #reduces predictor complexity while capturing as much variance as possible
    
    # Sum-of-the-parts forecast
    
    FC_OTHER[t,6]=unlist(mean(
      E_growth[(R+(t-1)-MA_SOP+1):(R+(t-1)), 1]) + DP_SOP[(R+(t-1)), 1]-r_f[R+(t-1), 1])#comibining several components
    if (FC_OTHER[t, 6] < 0){
      FC_OTHER_CT[t, 6] = 0
    }else{
      FC_OTHER_CT[t, 6] = FC_OTHER[t, 6];
    }
  }
  #non negativity
  # Integrating multiple forecast elements into a single predictive figure
}


# Forecast evaluation based on asset allocation exercise, 1986:1-2022:4 #


actual=Y[(R+P_0+1):nrow(Y), 1];
FC_HA=FC_HA[(P_0+1):nrow(FC_HA)];
FC_ECON=FC_ECON[(P_0+1):nrow(FC_ECON),];
FC_ECON_CT=FC_ECON_CT[(P_0+1):nrow(FC_ECON_CT),];
FC_OTHER=FC_OTHER[(P_0+1):nrow(FC_OTHER),];
FC_OTHER_CT=FC_OTHER_CT[(P_0+1):nrow(FC_OTHER_CT),];
#Focus only on the out-of-sample period that follows the reserved data used for training and validation

REC = read_xlsx('dataset.xlsx', sheet = 'Feuil1', range = 's146:s149', col_names = FALSE) #recession dummies, 1986:1-2022:4
#1 for recession, 0 for non-recession

EXP=-1*(REC-matrix(rep(1, nrow(REC),ncol = 1)));#subtracting the recession dummy from 1 and the multiplying by -1
#allow to identify expansion periods
index_EXP=which(EXP[, 1] != 0);
index_REC=which(REC[, 1] != 0);

gamma_MV=5; #coefficient of relative risk aversion, higher gamma implies greater risk aversion
window_VOL=4*5; # window size for estimating volatility, 20 quarters
FC_VOL=matrix(0, nrow = P,ncol = 1) #initialize matrix to store the estimated volatilties


   # Mean-Variance optimization (MVO) principles #
for (t in 1:P){
FC_VOL[t]=mean(Y[(R+P_0+(t-1)-window_VOL+1):(R+P_0+(t-1)), 1]^2)-(
  mean(Y[(R+P_0+(t-1)-window_VOL+1):(R+P_0+(t-1)), 1]))^2;
}
#calculating estimated volatility for each period in the evaluation phase using a rolling window
#Computes the variance of the actual returns
r_f_lag_P=r_f_lag[(R+P_0+1):(R+P_0+P), 1];
U_HA=matrix(0, nrow = 3, ncol = 1);
U_HA[1] = Perform_asset_allocation(actual,r_f_lag_P,FC_HA,FC_VOL,gamma_MV)[[1]]#asset allocation function
w_HA = Perform_asset_allocation(actual,r_f_lag_P,FC_HA,FC_VOL,gamma_MV)[[2]]#captures the utility value for the entire period

#utility calculation during expansions
U_HA[2] = Perform_asset_allocation(
  actual[index_EXP], r_f_lag_P[index_EXP, 1], FC_HA[index_EXP], FC_VOL[index_EXP], gamma_MV)[[1]]
xxx = Perform_asset_allocation(
  actual[index_EXP], r_f_lag_P[index_EXP, 1], FC_HA[index_EXP], FC_VOL[index_EXP], gamma_MV)[[2]]

#utility calculation during recessions
U_HA[3] = Perform_asset_allocation(
  actual[index_REC], r_f_lag_P[index_REC, 1], FC_HA[index_REC], FC_VOL[index_REC] ,gamma_MV)[[1]]
xxx = Perform_asset_allocation(
  actual[index_REC], r_f_lag_P[index_REC, 1], FC_HA[index_REC], FC_VOL[index_REC] ,gamma_MV)[[2]]

U_ECON = matrix(0, ncol(FC_ECON),3)
U_ECON_CT = matrix(0, ncol(FC_ECON_CT),3)
w_ECON = matrix(0, P, ncol(FC_ECON))
w_ECON_CT = matrix(0, P, ncol(FC_ECON_CT))
#initializing matrices for storing values

for (i in 1:nrow(U_ECON)){
#loop through each econometric model, applying the asset allocation strategy to calculate utility values and optimal weights
  
  
#Overall

U_ECON[i,1] = Perform_asset_allocation( #utility calculation for unsrestricted models
  actual, r_f_lag_P, FC_ECON[,i], FC_VOL, gamma_MV)[[1]]
w_ECON[,i] = Perform_asset_allocation( #portfolio weights for unrestricted models
  actual, r_f_lag_P, FC_ECON[,i], FC_VOL, gamma_MV)[[2]]


U_ECON_CT[i,1] = Perform_asset_allocation(#utility calculation for Campbell-Thompson models
  actual, r_f_lag_P, FC_ECON_CT[,i], FC_VOL, gamma_MV)[[1]]
w_ECON_CT[,i] = Perform_asset_allocation( #portfolio weights for Campbell-Thompson models
  actual, r_f_lag_P, FC_ECON_CT[,i], FC_VOL, gamma_MV)[[2]]

#Expansion, limited to economic period in expansion

U_ECON[i,2] = Perform_asset_allocation(
  actual[index_EXP], r_f_lag_P[index_EXP, 1], FC_ECON[index_EXP,i], FC_VOL[index_EXP], gamma_MV)[[1]]
xxx = Perform_asset_allocation(
  actual[index_EXP], r_f_lag_P[index_EXP, 1], FC_ECON[index_EXP,i], FC_VOL[index_EXP], gamma_MV)[[2]]


U_ECON_CT[i,2] = Perform_asset_allocation(
  actual[index_EXP], r_f_lag_P[index_EXP, 1], FC_ECON_CT[index_EXP,i], FC_VOL[index_EXP], gamma_MV)[[1]]
xxx = Perform_asset_allocation(
  actual[index_EXP], r_f_lag_P[index_EXP, 1], FC_ECON_CT[index_EXP,i], FC_VOL[index_EXP], gamma_MV)[[2]]



#Recession, limited to economic period in recession

U_ECON[i,3] = Perform_asset_allocation(
  actual[index_REC], r_f_lag_P[index_REC, 1], FC_ECON[index_REC,i], FC_VOL[index_REC], gamma_MV)[[1]]
xxx = Perform_asset_allocation(
  actual[index_REC], r_f_lag_P[index_REC, 1], FC_ECON[index_REC,i], FC_VOL[index_REC], gamma_MV)[[2]]


U_ECON_CT[i,3] = Perform_asset_allocation(
  actual[index_REC], r_f_lag_P[index_REC, 1], FC_ECON_CT[index_REC,i], FC_VOL[index_REC], gamma_MV)[[1]]
xxx = Perform_asset_allocation(
  actual[index_REC], r_f_lag_P[index_REC, 1], FC_ECON_CT[index_REC,i], FC_VOL[index_REC], gamma_MV)[[2]]


}

delta_ECON = 400 * (U_ECON - kronecker(t(U_HA), matrix(1, nrow = ncol(FC_ECON), ncol = 1)));
delta_ECON_CT = 400 * (U_ECON_CT - kronecker(t(U_HA), matrix(1, nrow = ncol(FC_ECON_CT), ncol = 1)));
#Calculate the utility difference (delta) between each econometric model and the historical average, by 400 to annualize the result
#kronecker expands the historical average across all econometric models
#the subtraction determines the performance improvement or degradation of each model against historical average
U_OTHER = matrix(0, ncol(FC_OTHER),3)
U_OTHER_CT = matrix(0, ncol(FC_OTHER_CT),3)
w_OTHER = matrix(0, P, ncol(FC_OTHER))
w_OTHER_CT = matrix(0, P, ncol(FC_OTHER_CT))
#initialize matrices to store utility values
for (i in 1:nrow(U_OTHER)){

#Overall

U_OTHER[i, 1] = Perform_asset_allocation(#utility calculation for unrestricted models
  actual, r_f_lag_P, FC_OTHER[,i], FC_VOL,gamma_MV)[[1]]
w_OTHER[ ,i] = Perform_asset_allocation(#portfolio weights for unrestricted models
  actual, r_f_lag_P, FC_OTHER[,i], FC_VOL,gamma_MV)[[2]]

U_OTHER_CT[i,1] = Perform_asset_allocation(#utility calculation for Campbell-Thompson models
  actual, r_f_lag_P, FC_OTHER_CT[,i], FC_VOL, gamma_MV)[[1]]
w_OTHER_CT[,i] = Perform_asset_allocation(#portfolio weights for Campbell-Thompson models
  actual, r_f_lag_P, FC_OTHER_CT[,i], FC_VOL, gamma_MV)[[2]]

#Expansion, during economic time of expansion

 U_OTHER[i,2] = Perform_asset_allocation(
   actual[index_EXP], r_f_lag_P[index_EXP, 1], FC_OTHER[index_EXP,i], FC_VOL[index_EXP], gamma_MV)[[1]]
xxx = Perform_asset_allocation(
  actual[index_EXP], r_f_lag_P[index_EXP, 1], FC_OTHER[index_EXP,i], FC_VOL[index_EXP], gamma_MV)[[2]]


U_OTHER_CT[i,2] = Perform_asset_allocation(
  actual[index_EXP], r_f_lag_P[index_EXP, 1], FC_OTHER_CT[index_EXP,i], FC_VOL[index_EXP], gamma_MV)[[1]]

xxx = Perform_asset_allocation(
  actual[index_EXP], r_f_lag_P[index_EXP, 1], FC_OTHER_CT[index_EXP,i], FC_VOL[index_EXP], gamma_MV)[[2]]

#Recession, during economic time of recession

U_OTHER[i,3] = Perform_asset_allocation(
  actual[index_REC], r_f_lag_P[index_REC, 1], FC_OTHER[index_REC,i], FC_VOL[index_REC], gamma_MV)[[1]]
xxx = Perform_asset_allocation(
  actual[index_REC], r_f_lag_P[index_REC, 1], FC_OTHER[index_REC,i], FC_VOL[index_REC], gamma_MV)[[2]]


U_OTHER_CT[i,3] = Perform_asset_allocation(
  actual[index_REC], r_f_lag_P[index_REC, 1], FC_OTHER_CT[index_REC,i], FC_VOL[index_REC], gamma_MV)[[1]]
xxx = Perform_asset_allocation(
  actual[index_REC], r_f_lag_P[index_REC, 1], FC_OTHER_CT[index_REC,i], FC_VOL[index_REC], gamma_MV)[[2]]
}

delta_OTHER = 400 * (U_OTHER - kronecker(t(U_HA), matrix(1, ncol(FC_OTHER), 1)));
delta_OTHER_CT = 400 * (U_OTHER_CT - kronecker(t(U_HA), matrix(1, ncol(FC_OTHER_CT) ,1)));
#Calculate the utility difference (delta) between each econometric model and the historical average, by 400 to annualize the result
#kronecker expands the historical average across all econometric models
#the subtraction determines the performance improvement or degradation of each model against historical average

# %save('Forecasts_quarterly_store','actual','FC_HA','FC_ECON','beta_ECON',...
#       %    'FC_ECON_CT','FC_OTHER','FC_OTHER_CT','FC_VOL','w_HA','w_ECON',...
#       %    'w_ECON_CT','w_OTHER','w_OTHER_CT','delta_ECON','delta_ECON_CT',...
#       %    'delta_OTHER_CT');


 # Writing results to spreadsheet #

Unrestricted <- data.frame(cbind(c(delta_ECON[, 1], delta_OTHER[, 1]) ,c( delta_ECON[, 2], delta_OTHER[, 2]), c(delta_ECON[, 3], delta_OTHER[, 3])))
colnames(Unrestricted) <- c('Overall', 'Expansion', 'Recession')
rownames(Unrestricted) <- c("log(DP)", "log(DY)", "log(EP)", "log(DE)", "SVAR", "BM", "NTIS", "TBL", "LTY", "LTR", "TMS", "DFY", "DFR", "INFL", "Kitchen sink", "SIC", "POOL-AVG", "POOL-DMSPE", "Diffusion index", "Sum-of-the-parts")
write.xlsx(Unrestricted, 'Delta_unrestricted_quarterly_2002_2022.xlsx', rowNames = TRUE)

Campbel_Tompson <- data.frame(cbind(c(delta_ECON_CT[,1], delta_OTHER_CT[,1]), 
                                    c(delta_ECON_CT[,2], delta_OTHER_CT[,2]),
                                    c(delta_ECON_CT[,2], delta_OTHER_CT[,2])))
colnames(Campbel_Tompson) <- c('Overall', 'Expansion', 'Recession')
rownames(Campbel_Tompson) <- c("log(DP)", "log(DY)", "log(EP)", "log(DE)", "SVAR", "BM", "NTIS", "TBL", "LTY", "LTR", "TMS", "DFY", "DFR", "INFL", "Kitchen sink", "SIC", "POOL-AVG", "POOL-DMSPE", "Diffusion index", "Sum-of-the-parts")
write.xlsx(Campbel_Tompson, 'Delta_CT_quarterly_2002_2022.xlsx', rowNames = TRUE)


HER_CT[index_REC,i], FC_VOL[index_REC], gamma_MV)[[2]]
}

delta_OTHER = 400 * (U_OTHER - kronecker(t(U_HA), matrix(1, ncol(FC_OTHER), 1)));
delta_OTHER_CT = 400 * (U_OTHER_CT - kronecker(t(U_HA), matrix(1, ncol(FC_OTHER_CT) ,1)));
#Calculate the utility difference (delta) between each econometric model and the historical average, by 400 to annualize the result
#kronecker expands the historical average across all econometric models
#the subtraction determines the performance improvement or degradation of each model against historical average

# %save('Forecasts_quarterly_store','actual','FC_HA','FC_ECON','beta_ECON',...
#       %    'FC_ECON_CT','FC_OTHER','FC_OTHER_CT','FC_VOL','w_HA','w_ECON',...
#       %    'w_ECON_CT','w_OTHER','w_OTHER_CT','delta_ECON','delta_ECON_CT',...
#       %    'delta_OTHER_CT');


 # Writing results to spreadsheet #

Unrestricted <- data.frame(cbind(c(delta_ECON[, 1], delta_OTHER[, 1]) ,c( delta_ECON[, 2], delta_OTHER[, 2]), c(delta_ECON[, 3], delta_OTHER[, 3])))
colnames(Unrestricted) <- c('